# Following page 72 of Professional Plone Development by Martin Aspeli

from Products.Five import zcml
from Products.Five import fiveconfigure
from Testing import ZopeTestCase as ztc
from Products.PloneTestCase import PloneTestCase as ptc
from Products.PloneTestCase.layer import onsetup

@onsetup
def setup_ContentWellPortlets():
    '''
    The @onsetup decorator means this will not execute until the 
    Plone site testing layer has been set up
    '''
    
    # Load the ZCML configuration
    fiveconfigure.debug_mode = True
    import Products.ContentWellPortlets
    zcml.load_config('configure.zcml', Products.ContentWellPortlets)
    fiveconfigure.debug_mode = False
    
    # Make the product available. 
    # Note: ztc.installPackage('Products.ContentWellPortlets') yielded "Installing Products.ContentWellPortlets ... NOT FOUND" 
    # Therefore did it this way instead
    from OFS.Application import install_package
    app = ztc.app()
    install_package(app, Products.ContentWellPortlets, Products.ContentWellPortlets.initialize)

# Call the (deferred) function
setup_ContentWellPortlets()
    
# Let Plone test case set up this product
ptc.setupPloneSite(products=['Products.ContentWellPortlets'])
    
class ContentWellPortletsTestCase(ptc.PloneTestCase):
    '''Use this base class for all the tests in the package. Put common setup code in here'''



# Following page 73 of Professional Plone Development by Martin Aspeli
# Requires that the base.py file exists

import unittest
from Products.ContentWellPortlets.tests.base import ContentWellPortletsTestCase

class TestSetup(ContentWellPortletsTestCase):
    '''
    Check whether product is installed correctly
    '''
    
    def afterSetUp(self):
        pass

    def testCSSregistered(self):
        '''
        Is our stylesheet registered?
        '''
        self.failUnless('++resource++ContentWellPortlets.styles/ContentWellPortlets.css' in self.portal.portal_css.getResourceIds(), 'Cannot find CSS')
        
    def testInterfaceAvailable(self):
        '''
        Is our product-specific interface available?
        '''
        from plone.browserlayer import utils
        from Products.ContentWellPortlets.browser.interfaces import IContentWellPortlets
        self.failUnless(IContentWellPortlets in utils.registered_layers(), 'Cannot find IContentWellPortlets interface')
    
    def testPortletManagers(self):
        '''
        Are our portlet managers available? Test by inserting a calendar portlet
        '''
        from zope.component import getUtility, getMultiAdapter
        from plone.app.portlets.portlets import calendar
        from plone.portlets.interfaces import IPortletManager
        from plone.portlets.interfaces import IPortletRenderer

        # get the portlet managers we should have created
        managerAbove = getUtility(IPortletManager, name='ContentWellPortlets.AbovePortletManager1',context=self.portal)
        managerBelow = getUtility(IPortletManager, name='ContentWellPortlets.BelowPortletManager1',context=self.portal)
        managerFooter = getUtility(IPortletManager, name='ContentWellPortlets.FooterPortletManager1',context=self.portal)
        
        # try rendering a portlet with it using getMultiAdapter((context, request, view, manager, assignment), Interface)
        renderer = getMultiAdapter((self.folder, self.folder.REQUEST, self.folder.restrictedTraverse('@@plone'), managerAbove, calendar.Assignment()), IPortletRenderer)
        self.failUnless(isinstance(renderer, calendar.Renderer), 'Cannot render portlet above contents')
        
        renderer = getMultiAdapter((self.folder, self.folder.REQUEST, self.folder.restrictedTraverse('@@plone'), managerBelow, calendar.Assignment()), IPortletRenderer)
        self.failUnless(isinstance(renderer, calendar.Renderer), 'Cannot render portlet below contents')
        
        renderer = getMultiAdapter((self.folder, self.folder.REQUEST, self.folder.restrictedTraverse('@@plone'), managerFooter, calendar.Assignment()), IPortletRenderer)
        self.failUnless(isinstance(renderer, calendar.Renderer), 'Cannot render footer portlet contents')
        
def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestSetup))
    return suite



from zope.interface import Interface
from plone.portlets.interfaces import IPortletManager
from plone.app.portlets.interfaces import IColumn

class IContentWellPortlets(Interface):
    """
    A layer specific to this product. Is registered using browserlayer.xml
    """

class IContentWellPortletManager(IPortletManager, IColumn):
    """
    Superclass used by our adapter
	The IColumn bit means that we can add all the portlets available to 
	 the right-hand and left-hand column portlet managers
    """
    
class IPortletsAboveContent(IContentWellPortletManager):
	 """
	 For the portlet manager above the content area.
	 """

class IPortletsBelowContent(IContentWellPortletManager):
	 """
	 For the portlet manager below the content area.
	 """

class IFooterPortlets(IContentWellPortletManager):
	 """
	 For the portlet manager in the footer area.
	 """



from zope.component import adapts
from zope.interface import Interface
from zope.publisher.interfaces.browser import IBrowserView
from zope.publisher.interfaces.browser import IDefaultBrowserLayer
from plone.app.portlets.manager import ColumnPortletManagerRenderer
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile

from Products.ContentWellPortlets.browser.interfaces import IContentWellPortletManager

class ContentWellPortletRenderer(ColumnPortletManagerRenderer):
    """
    A renderer for the content-well portlets
    """
    adapts(Interface, IDefaultBrowserLayer, IBrowserView, IContentWellPortletManager)
    template = ViewPageTemplateFile('templates/renderer.pt')



from zope.component import getMultiAdapter
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from plone.app.layout.viewlets.common import ViewletBase
from Products.CMFCore.utils import getToolByName

class PortletsAboveViewlet(ViewletBase):
    render = ViewPageTemplateFile('templates/portletsabovecontent.pt')
        
    def update(self):
        """
        Define everything we want to call in the template
        """
        context_state = getMultiAdapter((self.context, self.request), name=u'plone_context_state')
        self.manageUrl =  '%s/@@manage-portletsabovecontent' % context_state.view_url()
        
        ## This is the way it's done in plone.app.portlets.manager, so we'll do the same
        mt = getToolByName(self.context, 'portal_membership')
        self.canManagePortlets = mt.checkPermission('Portlets: Manage portlets', self.context)


class PortletsBelowViewlet(ViewletBase):
    render = ViewPageTemplateFile('templates/portletsbelowcontent.pt')
        
    def update(self):
        """
        Define everything we want to call in the template
        """
        context_state = getMultiAdapter((self.context, self.request), name=u'plone_context_state')
        self.manageUrl =  '%s/@@manage-portletsbelowcontent' % context_state.view_url()
        
        ## This is the way it's done in plone.app.portlets.manager, so we'll do the same
        mt = getToolByName(self.context, 'portal_membership')
        self.canManagePortlets = mt.checkPermission('Portlets: Manage portlets', self.context)


class FooterPortletsViewlet(ViewletBase):
    render = ViewPageTemplateFile('templates/footerportlets.pt')
        
    def update(self):
        """
        Define everything we want to call in the template
        """
        context_state = getMultiAdapter((self.context, self.request), name=u'plone_context_state')
        self.manageUrl =  '%s/@@manage-footerportlets' % context_state.view_url()
        
        ## This is the way it's done in plone.app.portlets.manager, so we'll do the same
        mt = getToolByName(self.context, 'portal_membership')
        self.canManagePortlets = mt.checkPermission('Portlets: Manage portlets', self.context)


# To run the tests for this product, cd to the directory where you would normally run this command:
# bin/instance fg
# Instead, make sure Zope is not running, and type
# bin/instance test -s Products.ContentWellPortlets